<html>

<body>
	
	{% if entered %}
		<p>You entered: <blockquote>{{ entered }}</blockquote></p>
	{% endif %}
	
	<form action="{% url search_form %}" method="GET">
	<table>
	{{ form }}
	<tr>
		<th></th><td><input type="submit" /></td>
	</tr>
	</table>
	</form>
	
</body>
</html>